/*******************************************************************************
 * Copyright (c) 2004 Elias Volanakis.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*Elias Volanakis - initial API and implementation
 *    Chris Aniszczyk - updated to work with EMF
 *    IBM Corporation
*******************************************************************************/

package org.eclipse.gef.examples.shapes.emf.parts;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;
import org.eclipse.gef.editpolicies.ConnectionEditPolicy;
import org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy;
import org.eclipse.gef.examples.shapes.emf.model.Connection;
import org.eclipse.gef.examples.shapes.emf.model.ModelPackage;
import org.eclipse.gef.examples.shapes.emf.model.commands.ConnectionDeleteCommand;
import org.eclipse.gef.examples.shapes.emf.model.properties.ConnectionPropertySource;
import org.eclipse.gef.requests.GroupRequest;
import org.eclipse.ui.views.properties.IPropertySource;

/**
 * Edit part for Connection model elements.
 * <p>This edit part must implement the PropertyChangeListener interface, 
 * so it can be notified of property changes in the corresponding model element.
 * </p>
 * @author Elias Volanakis
 */
class ConnectionEditPart extends AbstractConnectionEditPart {
	
	private ConnectionAdapter adapter = new ConnectionAdapter(this);
	
	private class ConnectionAdapter implements Adapter {
		
		private Notifier target;
		private ConnectionEditPart editPart;
		
		public ConnectionAdapter(ConnectionEditPart part) {
			this.editPart = part;
		}
		
		/**
		 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
		 */
		public void notifyChanged(Notification notification) {
			int type = notification.getEventType();
			int featureId = notification.getFeatureID(ModelPackage.class);
			if(type == Notification.SET) {
				switch(featureId) {
				case ModelPackage.CONNECTION__DASHED:
					if(getCastedModel().isDashed()) {
						((PolylineConnection) getFigure()).setLineStyle(Graphics.LINE_DASH);
					}
					else {
						((PolylineConnection) getFigure()).setLineStyle(Graphics.LINE_SOLID);
					}
					refreshVisuals();
					break;
				}
			}
		}
		
		/**
		 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
		 */
		public void setTarget(Notifier newTarget) {
			target = newTarget;
		}
		
		/**
		 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
		 */
		public boolean isAdapterForType(Object type) {
			return (getModel().getClass() == type);
		}
		
		/**
		 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
		 */
		public Notifier getTarget() {
			return target;
		}
		
	}
	
	/**
	 * Upon activation, attach to the model element as a property change listener.
	 */
	public void activate() {
		if (!isActive()) {
			hookIntoModel(getCastedModel());
			super.activate();
		}
	}
	
	private void hookIntoModel(EObject model) {
		if(model != null) {
			model.eAdapters().add(adapter);
		}
	}
	
	private void unhookFromModel(EObject model) {
		if(model != null) {
			model.eAdapters().remove(adapter);
		}
	}
	
	public Object getAdapter(Class key) {
		if (IPropertySource.class == key)
			return new ConnectionPropertySource(getModel());
		return super.getAdapter(key);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// Selection handle edit policy. 
		// Makes the connection show a feedback, when selected by the user.
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE,
				new ConnectionEndpointEditPolicy());
		// Allows the removal of the connection model element
		installEditPolicy(EditPolicy.CONNECTION_ROLE, new ConnectionEditPolicy() {
			protected Command getDeleteCommand(GroupRequest request) {
				return new ConnectionDeleteCommand(getCastedModel());
			}
		});
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		PolylineConnection connection = (PolylineConnection) super.createFigure();
		connection.setTargetDecoration(new PolygonDecoration()); // arrow at target endpoint
		// this check is kind of stupid, I really should be using an enumerated
		// type for the line style that matches Graphics :/
		if(getCastedModel().isDashed()) {
			connection.setLineStyle(Graphics.LINE_DASH);
		}
		else {
			connection.setLineStyle(Graphics.LINE_SOLID);
		}
		return connection;
	}
	
	/**
	 * Upon deactivation, detach from the model element as a property change listener.
	 */
	public void deactivate() {
		if (isActive()) {
			unhookFromModel(getCastedModel());
			super.deactivate();
		}
	}
	
	private Connection getCastedModel() {
		return (Connection) getModel();
	}
	
}